/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 * Description:
 *      Chunk info handler.
 * Create: 2022-01-23
 * Notes: NA
 */

#ifndef CODE_RTFCHUNKMANAGER_H
#define CODE_RTFCHUNKMANAGER_H

#include <memory>
#include <queue>

#include "ara/core/map.h"
#include "ara/core/string.h"
#include "ara/core/vector.h"

namespace rtf {

struct OutMessage {
    OutMessage() : msgBuff(nullptr), buffLen(0U), keyValueLen(0U)
    {}
    ara::core::Vector<ara::core::String> fieldName;
    ara::core::Vector<ara::core::String> fieldValue;
    uint8_t*                             msgBuff;
    uint32_t                             buffLen;
    uint32_t                             keyValueLen;
};

struct OutConnx {
    OutConnx() : keyValueLen(0U)
    {}
    ara::core::Vector<ara::core::String> fieldName;
    ara::core::Vector<ara::core::String> fieldValue;
    uint32_t                             keyValueLen;
};

struct OutChunkHeader {
    OutChunkHeader() : keyValueLen(0U)
    {}
    ara::core::Vector<ara::core::String> fieldName;
    ara::core::Vector<ara::core::String> fieldValue;
    uint32_t                             keyValueLen;
};

class RtfChunkManager {
public:
    RtfChunkManager() noexcept;
    ~RtfChunkManager();
    RtfChunkManager(RtfChunkManager const&) = default;
    RtfChunkManager(RtfChunkManager&&) = default;
    RtfChunkManager& operator=(RtfChunkManager const&) = default;
    RtfChunkManager& operator=(RtfChunkManager&&) = default;
    bool SetChunkHeaderRecord(ara::core::Vector<ara::core::String> const& fieldName,
                              ara::core::Vector<ara::core::String> const& fieldValue);
    uint32_t GetChunkHeaderRecordLength() const noexcept;
    bool SetMessageRecord(ara::core::Vector<ara::core::String> const& fieldName,
                          ara::core::Vector<ara::core::String> const& fieldValue,
                          uint8_t* buf,
                          uint32_t length);

    bool SetChunkIndex(ara::core::Vector<ara::core::String> const& fieldName,
                       ara::core::Vector<ara::core::String> const& fieldValue,
                       uint8_t* buf,
                       uint32_t length);
    uint32_t GetChunkIndexSize() const noexcept;

    bool SetTempChunkInfo(ara::core::Vector<ara::core::String> const& fieldName,
                          ara::core::Vector<ara::core::String> const& fieldValue,
                          uint8_t* buf,
                          uint32_t length);
    uint32_t GetTempChunkInfoSize() const noexcept;

    uint32_t GetChunkSize() const noexcept;

    bool SetTempConnectionRecord(ara::core::Vector<ara::core::String> const& fieldName,
                                 ara::core::Vector<ara::core::String> const& fieldValue);

    uint32_t GetTempConnectionRecordLength() const noexcept;

    void GetChunk(OutChunkHeader& chunkHeader,
                  std::queue<OutMessage>& queueBuff,
                  std::queue<OutMessage>& queueChunkIndex,
                  OutMessage& tempChunkInfo,
                  std::queue<OutConnx>& queueConnx);
    std::queue<OutMessage> GetChunkMessages();

    bool SetcurrentChunkNewEventCount(const uint32_t currentChunkNewEventCount) noexcept;

    uint32_t GetcurrentChunkNewEventCount() const noexcept;
    uint32_t GetRecordHeaderLen(ara::core::Vector<ara::core::String> const& fieldName,
                                ara::core::Vector<ara::core::String> const& fieldValue) const noexcept;

    void DeleteResource() noexcept;
private:
    OutChunkHeader chunkHeader_;
    std::queue<OutMessage> queueBuff_;
    std::queue<OutMessage> queueChunkIndex_;
    OutMessage tempChunkInfo_;
    std::queue<OutConnx> queueConnx_;

    uint32_t chunkSize_;
    uint32_t chunkIndexSize_;
    uint32_t tempChunkInfoSize_;
    uint32_t connxSize_;
    uint32_t currentChunkNewEventCount_;
};
} // end of namespace rtf
#endif
