/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2019. All rights reserved.
 * Description:
 *      This file is the implement of class RtfMsgEntity.
 *      MessageEntity is deisgned to store message
 * Create: 2019-11-30
 * Notes: NA
 */
#ifndef RTF_MSG_ENTITY_H
#define RTF_MSG_ENTITY_H

#include <functional>
#include <set>

#include "ara/core/string.h"
#include "ara/core/vector.h"
#include "rtf/internal/RtfBagFile.h"
#include "rtf/internal/RtfBagStructs.h"
#include "rtf/internal/RtfBuffer.h"

namespace rtf {
namespace rtfbag {
struct RtfTrueQuery {
    bool operator()(Connection const&) const
    {
        return true;
    }
};

struct RtfEventQuery {
    explicit RtfEventQuery(ara::core::String const& event);
    explicit RtfEventQuery(ara::core::Vector<ara::core::String> const& events,
                           ara::core::Vector<ara::core::String> const& skipEvents = {},
                           ara::core::String const& network = "",
                           uint16_t domainId = RTF_DEFAULT_DOMAINID);
    bool IsSkipEvent(Connection const& info) const noexcept;
    bool CheckEventIndex(Connection const& info) const noexcept;

    bool operator()(Connection const& info) const;
    ara::core::Vector<ara::core::String> events_;
    ara::core::Vector<ara::core::String> skipEvents_;
    ara::core::String networkIP_;
    uint16_t domainId_{RTF_DEFAULT_DOMAINID};
};

struct RtfQuery {
    RtfQuery(std::function<bool(Connection const&)> const& query, uint64_t const startTime, uint64_t const endTime);

    std::function<bool(Connection const&)> query_;
    uint64_t startTime_;
    uint64_t endTime_;
};

struct RtfBagQuery {
    RtfBagQuery(RtfBagFile & bag, RtfQuery const& query) noexcept;

    RtfBagFile* bag_;
    RtfQuery query_;
};

struct MessageRange	{
    MessageRange(std::multiset<MessageIndex>::const_iterator const begin,
        std::multiset<MessageIndex>::const_iterator const& end,
        Connection const& connection, RtfBagQuery& bagQuery) noexcept;
    std::multiset<MessageIndex>::const_iterator begin_;
    std::multiset<MessageIndex>::const_iterator end_;
    Connection const *connection_;
    RtfBagQuery* bagQuery_;
};

struct ViewIterHelper {
    ViewIterHelper(std::multiset<MessageIndex>::const_iterator const iter, MessageRange& range) noexcept;
    std::multiset<MessageIndex>::const_iterator iter_;
    MessageRange *range_;
};

struct ViewIterHelpCompare {
    bool operator()(ViewIterHelper const& first, ViewIterHelper const& second) const;
};

class RtfMsgEntity {
public:
    RtfMsgEntity(Connection const& conn, MessageIndex const& msgIndex, RtfBagFile& bag) noexcept;
    ~RtfMsgEntity();
    RtfMsgEntity(RtfMsgEntity const&) = default;
    RtfMsgEntity(RtfMsgEntity&&) = default;
    RtfMsgEntity& operator=(RtfMsgEntity const&) = default;
    RtfMsgEntity& operator=(RtfMsgEntity&&) = default;

    FileErrorCode WriteMsg(RtfBuffer& buffer, const std::uint32_t len = 0U) const;
    ara::core::String GetEvent() const;
    ara::core::String GetRecordingEvent() const;
    ara::core::String GetDataType() const;
    uint64_t GetTime() const noexcept;
    bool GetMsgSize(uint32_t& size) const;
    const Connection& GetConnection() const noexcept;
    uint32_t GetBagVersion() const;

private:
    Connection const *conn_;
    MessageIndex const *msgIndex_;
    RtfBagFile* bag_;
};
}  // namespace rtfbag
}  // namespace rtf
#endif
