/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2019. All rights reserved.
 * Description: implement of class RtfPubClock
 *              RtfPubClock time clock for rtfbag palyer to public events
 * Create: 2019-12-17
 */
#ifndef RTF_PUB_CLOCK_H
#define RTF_PUB_CLOCK_H

#include <chrono>

#include "rtf/internal/RtfTimeTranslator.h"

namespace rtf {
namespace rtfbag {
class RtfPubClock {
public:
    RtfPubClock() = default;

    void SetTimeRate(const double rate) noexcept;
    void SetBagHorizon(const std::uint64_t horizon) noexcept;
    void SetSysHorizon(const MilliTimePoint horizon) noexcept;
    void SetCurrentBagTime(const std::uint64_t time) noexcept;
    std::uint64_t GetTime() const noexcept;

    // run duration until sysHorizon_ is reached
    void RunClock(const std::chrono::milliseconds duration);

    // step to horizon time
    void StepClock() noexcept;

    bool HorizonReached() const noexcept;

private:
    double rate_{ 1.0 };                        // rate of sys_time/bag_time
    MilliTimePoint sysHorizon_;                 // horizon sys time
    uint64_t bagHorizon_{ 0U };                 // horizon bag time
    uint64_t currentBagTime_{ 0U };             // current bag timestamp
};
}  // namespace rtfbag
}  // namespace rtf
#endif
