/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2019. All rights reserved.
 * Description:
 *      This file is the implement of class RtfView.
 *      RtfView is for publish messages
 * Create: 2019-11-30
 * Notes: NA
 */
#ifndef RTF_VIEW_H
#define RTF_VIEW_H

#include <memory>

#include "ara/core/string.h"
#include "ara/core/vector.h"
#include "rtf/internal/RtfBagFile.h"
#include "rtf/internal/RtfBagStructs.h"
#include "rtf/internal/RtfMsgEntity.h"

namespace rtf {
namespace rtfbag {
constexpr uint64_t TIME_MIN = 0U;
constexpr uint64_t TIME_MAX = UINT64_MAX;

class RtfView {
public:
    class Iterator {
        friend class RtfView;
    public:
        explicit Iterator(RtfView const& view);
        ~Iterator();
        Iterator(Iterator const&) = default;
        Iterator(Iterator&&) = default;
        Iterator& operator=(Iterator const &) = default;
        Iterator& operator=(Iterator&&) = default;

        Iterator& operator++()
        {
            static_cast<void>(Increase());
            return *this;
        }

        RtfMsgEntity& Value();
        std::uint64_t BeginTime();
        bool IsEnd() const noexcept;
        RtfMsgEntity GetPosValue(const std::uint32_t pos);
    protected:
        bool Populate();
        bool Increase() noexcept;
        RtfMsgEntity& Dereference();
    private:
        RtfView const *view_;
        std::uint32_t size_;
        std::shared_ptr<RtfMsgEntity> msgEntity_;
        ara::core::Vector<ViewIterHelper> iters_;
    };

    RtfView() = default;
    ~RtfView();
    RtfView(RtfView const&) = default;
    RtfView(RtfView&&) = default;
    RtfView& operator=(RtfView const&) = default;
    RtfView& operator=(RtfView&&) = default;

    Iterator Begin() const;

    void AddQuery(RtfBagFile& bag, std::uint64_t const startTime = TIME_MIN, std::uint64_t const endTime = TIME_MAX);
    void AddQuery(RtfBagFile& bag, std::function<bool(Connection const&)> const& query,
        std::uint64_t const startTime = TIME_MIN, std::uint64_t const endTime = TIME_MAX);
    ara::core::Vector<Connection const*> GetConnections();
    std::uint32_t Size() noexcept;
    std::uint64_t GetBeginTime() noexcept;
    std::uint64_t GetEndTime() noexcept;

protected:
    void UpdateQueries(RtfBagQuery& bagQuery);
    std::shared_ptr<RtfMsgEntity> CreateMsgEntity(Connection const& connection,
        MessageIndex const& msgIndex, RtfBagFile& bag) const;

private:
    ara::core::Vector<std::shared_ptr<MessageRange>> ranges_;
    ara::core::Vector<std::shared_ptr<RtfBagQuery>>  queries_;
    uint64_t                         size_ {0U};
};
}  // namespace rtfbag
}  // namespace rtf
#endif
