/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 * Description: ddt compressor
*/
#ifndef DDT_COMPRESSOR_H
#define DDT_COMPRESSOR_H

#include <cstdint>

namespace mdc {
namespace ddt {
enum class CompressionType : uint32_t {
    NONE = 0U,
    GZIP = 1U,
    ZLIB = 2U
};

enum class CompressorAction : uint32_t {
    COMPRESS = 0U,
    DECOMPRESS
};

enum class CompressorRetCode : uint32_t {
    RET_SUCCESS = 0U,
    RET_GZIP_COMPRESSOR_INIT_FAILED = 1U,
    RET_ZLIB_COMPRESSOR_INIT_FAILED = 2U,
    RET_GZIP_DECOMPRESSOR_INIT_FAILED = 3U,
    RET_ZLIB_DECOMPRESSOR_INIT_FAILED = 4U,
    RET_PARAM_ERROR = 5U,
    RET_NOT_SUPPORT_TYPE = 6U,
    RET_COMPRESS_ERROR = 7U,
    RET_COMPRESSOR_DEINIT_FAILED = 8U,
    RET_DECOMPRESSOR_DEINIT_FAILED = 9U,
    RET_MEMCPY_FAILED = 10U
};

class Compressor {
public:
    explicit Compressor(const CompressionType& type, const CompressorAction& action);
    virtual ~Compressor();
    int32_t TransferStream(const uint8_t* inStream, const uint64_t &inputLength,
                           uint8_t* &outStream, uint64_t &outputLength);
    int32_t Init();
    void DeInit() noexcept;

private:
    Compressor(const Compressor&) = delete;
    Compressor& operator=(const Compressor&) = delete;
    Compressor(Compressor&&) = delete;
    Compressor& operator=(Compressor&&) = delete;

    int32_t StreamInit();
    int32_t StreamEnd();
    int32_t CompressInit();
    int32_t DecompressInit();

    int32_t Compress(const uint8_t* inStream, const uint64_t &inputLength,
                     uint8_t* &outStream, uint64_t &outputLength);
    int32_t Decompress(const uint8_t* inStream, const uint64_t &inputLength,
                       uint8_t* &outStream, uint64_t &outputLength);

    CompressionType compressionType_;
    CompressorAction actionType_;
    void* stream_;
};
}
}
#endif
