/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2022. All rights reserved.
 */

#ifndef RTF_MAINTAIND_IMPL_TYPE_APSOMEIPTRANSFORMATIONPROPS_H
#define RTF_MAINTAIND_IMPL_TYPE_APSOMEIPTRANSFORMATIONPROPS_H
#include "rtf/stdtype/impl_type_uint8_t.h"
#include "rtf/maintaind/serialize/impl_type_byteorder.h"
#include "rtf/maintaind/serialize/impl_type_wiretype.h"

namespace rtf {
namespace maintaind {
enum class StringEncoding : rtf::stdtype::uint8_t {
    UTF_8,
    UTF_16
};
struct ApSomeipTransformationProps {
    ::rtf::stdtype::uint8_t alignment {1U};
    ::rtf::maintaind::ByteOrder byteorder {::rtf::maintaind::ByteOrder::BIGENDIAN};
    bool implementsLegencyStringSerialization {false};
    ::rtf::maintaind::WireType wireType {::rtf::maintaind:: WireType::STATIC};
    ::rtf::stdtype::uint8_t arrayLengthField {0U};
    ::rtf::stdtype::uint8_t mapLengthField {4U};
    ::rtf::stdtype::uint8_t vectorLengthField {4U};
    ::rtf::stdtype::uint8_t stringLengthField {4U};
    ::rtf::stdtype::uint8_t structLengthField {0U};
    ::rtf::stdtype::uint8_t unionLengthField {4U};
    ::rtf::stdtype::uint8_t unionTypeSelectorLength {4U};
    ::rtf::maintaind::StringEncoding stringEncoding {::rtf::maintaind::StringEncoding::UTF_8};

    static bool IsPlane() noexcept { return true; }

    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun) noexcept
    {
        fun(alignment);
        fun(byteorder);
        fun(implementsLegencyStringSerialization);
        fun(wireType);
        fun(arrayLengthField);
        fun(mapLengthField);
        fun(vectorLengthField);
        fun(stringLengthField);
        fun(structLengthField);
        fun(unionLengthField);
        fun(unionTypeSelectorLength);
        fun(stringEncoding);
    }

    template<typename F>
    void enumerate(F& fun) const noexcept
    {
        fun(alignment);
        fun(byteorder);
        fun(implementsLegencyStringSerialization);
        fun(wireType);
        fun(arrayLengthField);
        fun(mapLengthField);
        fun(vectorLengthField);
        fun(stringLengthField);
        fun(structLengthField);
        fun(unionLengthField);
        fun(unionTypeSelectorLength);
        fun(stringEncoding);
    }

    friend bool operator==(
        const ::rtf::maintaind::ApSomeipTransformationProps& lhs, const ::rtf::maintaind::ApSomeipTransformationProps& rhs) noexcept
    {
        return ((lhs.alignment == rhs.alignment) && (lhs.byteorder == rhs.byteorder) &&
        (lhs.implementsLegencyStringSerialization == rhs.implementsLegencyStringSerialization) &&
        (lhs.wireType == rhs.wireType) &&
        (lhs.arrayLengthField == rhs.arrayLengthField) && (lhs.mapLengthField == rhs.mapLengthField) &&
        (lhs.vectorLengthField == rhs.vectorLengthField) && (lhs.stringLengthField == rhs.stringLengthField) &&
        (lhs.structLengthField == rhs.structLengthField) && (lhs.unionLengthField == rhs.unionLengthField) &&
        (lhs.unionTypeSelectorLength == rhs.unionTypeSelectorLength) && (lhs.stringEncoding == rhs.stringEncoding));
    }
};
} // namespace maintaind
} // namespace rtf


#endif // RTF_MAINTAIND_IMPL_TYPE_APSOMEIPTRANSFORMATIONPROPS_H
