/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2019. All rights reserved.
 * Description:
 *      This file is the implement of class BagInfo.
 *      BagInfo will read a bag file and and provide info for usr
 * Create: 2019-12-23
 * Notes: NA
 */
#ifndef RTF_BAG_INFO_H
#define RTF_BAG_INFO_H

#include <set>

#include "ara/core/map.h"
#include "ara/core/string.h"
#include "ara/core/vector.h"
#include "rtf/maintaind/impl_type_drivertype.h"

namespace rtf {
namespace rtfbag {
constexpr std::uint16_t RTF_DEFAULT_DOMAINID{1000U};
struct BagFileMessageInfo {
    ara::core::String message;
    ara::core::String types;
    uint32_t size;
    double freq;
    rtf::maintaind::DriverType driverType;
    uint16_t domainId{RTF_DEFAULT_DOMAINID};
    ara::core::String networkIP;
    ara::core::String udpMulticastAddr;
    ara::core::Vector<ara::core::String> discoveryMulticastAddr;
    bool isFSHMMulticast{false};
    ara::core::String fshmMulticastAddr;
};

struct BagFileHeadInfo {
    enum class CompressionType : std::uint32_t {
        NONE     = 0U,    // 未压缩
        GZIP     = 1U,    // GZIP
        ZLIB     = 2U,    // ZLIB
    };
    ara::core::String path;
    ara::core::String version;
    uint64_t startRecordRealTime;
    uint64_t startRecordVirtualTime;
    uint64_t stopRecordRealTime;
    uint64_t stopRecordVirtualTime;
    uint64_t duration;
    uint64_t start;
    uint64_t end;
    uint64_t size;
    uint32_t messages;
    bool isValid;
    ara::core::Map<ara::core::String, BagFileMessageInfo> messageHead;
    std::multimap<ara::core::String, BagFileMessageInfo> messageMultiHead;
    CompressionType compression;
};
class RtfBagInfo {
public:
    static void QueryBagInfo(const std::set<ara::core::String>& paths,
    ara::core::Map<ara::core::String, BagFileHeadInfo>& bagFileInfo);
};
}
}
#endif
