/*

 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2019. All rights reserved.

 * Description: rtf event API


 * Create: 2019-11-25

 * Notes: 无


 */

#ifndef RTFTOOLS_RTFEVENT_H
#define RTFTOOLS_RTFEVENT_H

#include <deque>
#include <chrono>
#include <numeric>
#include <mutex>
#include <set>
#include <functional>
#include <cmath>
#include <tuple>
#include <fstream>

#include "ara/core/vector.h"
#include "ara/core/string.h"
#include "ara/core/map.h"
#include "rtf/internal/tools_common_client_manager.h"
#include "json_parser/document.h"
#include "json_parser/global.h"
#include "rtf/com/rtf_com.h"
#include "rtf/internal/rtf_tools_type.h"
#include "rtf/common/serialize/serialize_type.h"
#include "rtf/rtfnode/RtfNode.h"
#include "json_parser/json_writer.h"

namespace rtf {
namespace rtftools{
namespace utils{
class NetProcess;
}
}
namespace rtfevent {
const ara::core::String DATA_TYPE_VALUE_UINT8 {"uint8_t"};
const ara::core::String DATA_TYPE_VALUE_UINT16 {"uint16_t"};
const ara::core::String DATA_TYPE_VALUE_UINT32 {"uint32_t"};
const ara::core::String DATA_TYPE_VALUE_UINT64 {"uint64_t"};
const ara::core::String DATA_TYPE_VALUE_INT8 {"int8_t"};
const ara::core::String DATA_TYPE_VALUE_INT16 {"int16_t"};
const ara::core::String DATA_TYPE_VALUE_INT32 {"int32_t"};
const ara::core::String DATA_TYPE_VALUE_INT64 {"int64_t"};
const ara::core::String DATA_TYPE_VALUE_STRING {"string"};
const ara::core::String DATA_TYPE_VALUE_FLOAT {"float"};
const ara::core::String DATA_TYPE_VALUE_DOUBLE {"double"};
const ara::core::String DATA_TYPE_STRUCT {"structure"};
const ara::core::String DATA_TYPE_VECTOR {"vector"};
const ara::core::String DATA_TYPE_ARRAY {"array"};
const ara::core::String DATA_TYPE_MAP {"associative_map"};
const ara::core::String DATA_TYPE_ENUMERATION {"enumeration"};
const ara::core::String DATA_TYPE_VARIANT {"variant"};

using DataTypeReturnValue = rtf::DataTypeReturnValue;
using EventShowReturnValue = rtf::ShowReturnValue;

class RtfEvent {
public:
    RtfEvent();
    ~RtfEvent() = default;

    RtfEvent(const RtfEvent& other) = default;
    RtfEvent& operator=(const RtfEvent& other) & = default;
    RtfEvent(RtfEvent&& other) = default;
    RtfEvent& operator=(RtfEvent&& other) & = default;

    int32_t Query(const ara::core::String eventName,
                  RtfEventInfo &rtfEventInfo,
                  const std::unique_ptr<CommonFilter> filter = nullptr);

    int32_t Query(const ara::core::String eventName,
                  ara::core::Vector<RtfEventInfo> &rtfEventInfoList,
                  const std::unique_ptr<CommonFilter> filter = nullptr);

    /*******************************************************************************
      函 数 名		:  QueryAll
      功能描述		:  查询环境上存在的event信息，支持参数过滤功能
      输入参数		:  filter，过滤参数
      输出参数		:  rtfEventInfoList，环境上存在的event信息
      返 回 值		:  int32_t, 0表示查询成功，-1表示查询出错
    *******************************************************************************/
    int32_t QueryAll(ara::core::Vector<RtfEventInfo> &rtfEventInfoList,
                     const std::shared_ptr<EventFilter> &filter = nullptr);

    /*******************************************************************************
      函 数 名		:  QueryAllWithNamespace
      功能描述		:  查询环境上指定nameSpace下存在的event信息，支持参数过滤功能
      输入参数		:  filter，过滤参数
                      nameSpace, 指定命名空间
      输出参数		:  rtfEventInfoList，环境上存在的event信息
      返 回 值		:  int32_t, 0表示查询成功，-1表示查询出错
    *******************************************************************************/
    int32_t QueryAllWithNamespace(const ara::core::String nameSpace,
                              ara::core::Vector<RtfEventInfo> &rtfEventInfoList,
                              const std::shared_ptr<EventFilter> &filter = nullptr);
    int32_t QueryEventShowInfo(const ara::core::String eventName,
                           EventShowReturnValue &rtfEventDataType,
                           const std::unique_ptr<CommonFilter> filter = nullptr);
    int32_t QueryDataTypeInfo(const ara::core::String dataTypeName,
                            ara::core::Vector<DataTypeReturnValue> &rtfDataTypeMsg) const;
    int32_t Init();

    int32_t Export(const ara::core::String& fileName, const ara::core::String& path = "");
private:
    struct ExportNodeInfo {
        ara::core::String node;
        ara::core::String identity;
        ara::core::String seLinuxLabel;
    };

    struct ExportDdsInfo {
        ara::core::String topicName;
        uint64_t domain{0U};
        std::vector<ara::core::String> transportPlugin;
        bool isFSHMMulticast{false};
        ara::core::String fshmMulticastAddr;
    };

    struct ExportInfo {
        ara::core::String name;
        ResourceInfo resourceInfo;
        uint16_t serviceId{0U};
        ara::core::String instanceId;
        ara::core::String networkBinding;
        bool isMbuf{false};
        std::vector<ExportNodeInfo> subscribe;
        ExportNodeInfo publish;
        ExportDdsInfo dds;
    };

    int32_t QueryAllDataType(
        const ara::core::Vector<std::shared_ptr<rtf::rtftools::common::RTFMaintaindToolsServiceProxy>>& proxyList,
        const uint32_t waitForMs, const int32_t preResult) const;
    std::tuple<bool, EventShowReturnValue> GetShowResult(
        const maintaind::proxy::methods::QueryEventShow::Output& outPut,
        const EventShowReturnValue& rtfEventType) const;
    void SetRtfEventInfo(RtfEventInfo& rtfEventInfo,
        const rtf::maintaind::EventInfoWithPubSub& eventInfoWithPubSub) const;
    rtf::maintaind::QuerySubEventInfoIndex GetSubNodeIndex(
        const rtf::maintaind::EventInfoWithPubSub &eventInfoWithPubSub) const;
    int32_t QuerySubNodeInfo(ara::core::Vector<rtf::maintaind::EventInfoWithPubSub> &eventInfoWithPubSubList) const;
    int32_t GetExportInfo(ara::core::Vector<rtf::rtfevent::RtfEventInfo>& rtfEventInfoList,
                          ara::core::Vector<rtfnode::RtfNodeInfo>& rtfNodeInfoList);
    void GetEventInfo(const ara::core::Vector<rtf::rtfevent::RtfEventInfo>& rtfEventInfoList,
                      const ara::core::Vector<rtfnode::RtfNodeInfo>& rtfNodeInfoList);
    void GetEventInfoForDDS(const rtf::rtfevent::RtfEventInfo& rtfEventInfo, ExportInfo& info) const;
    ara::core::String ExportJsonInfo();
    void ExportEventInfo(ara::godel::common::jsonParser::JsonWriter& writer, const ExportInfo& info) const;
    void ExportPubInfo(ara::godel::common::jsonParser::JsonWriter& writer, const ExportInfo& info) const;
    void ExportSubInfo(ara::godel::common::jsonParser::JsonWriter& writer, const ExportInfo& info) const;
    void ExportSubNodeInfo(ara::godel::common::jsonParser::JsonWriter& writer, const ExportNodeInfo& nodeInfo) const;
    void ExportEventDdsInfo(ara::godel::common::jsonParser::JsonWriter& writer, const ExportInfo& info) const;
    bool isInit_ {false};
    std::vector<ExportInfo> exportInfo_;
    std::shared_ptr<rtf::rtftools::common::ToolsCommonClientManager> toolsCommonClientManager_ {nullptr};
    std::shared_ptr<rtftools::utils::NetProcess> netProcessPtr_;
};

// hz
struct EventHzInfo {
    double min_delta{0.0};   // 最小的时间间隔差。
    double max_delta{0.0};   // 最大的时间间隔差。
    double rate{0.0};        // 统计的平均频率，rate=num/sum。
    double std_dev{0.0};     // 所有时间标准差，标准差计算公式如下所示。
    uint32_t window{0U};    // 记录消息窗口个数，有最大缓存上限，需要配置初始值。
    bool publish{false};       // 是否发布对应的消息
    bool isSubed{false};       // sub是否建立成功
    std::string uuid;
};
struct DDSTopicConfig {
    ara::core::String topic;
};

struct HzOptions {
    ara::core::String ddsNet{""};
    ara::core::String someipNet{""};
    ara::core::String uuid{""};
    ara::core::String networkIP{""};
    std::uint16_t domainId{RTF_DEFAULT_DOMAINID};
};

enum class RtfeventHzType : uint8_t {
    ALL_RIGHT                 = 0U,    // 订阅消息
    WARN_NO_SOMEIP_NET        = 1U,    // 没有输入SOME/IP网卡network名称
    EROR_CREATE_SOMEIP        = 2U,    // SOMEIP sub创建失败
    EROR_CREATE_DDS           = 3U,    // DDS sub创建失败
    EROR_CREATE_ALL           = 4U,    // 所有event sub创建失败
};

class RtfEventHz {
public:
    using HzInfoCallback = std::function<void(const ara::core::Vector<std::pair<ara::core::String, EventHzInfo>>&,
                                              RtfeventHzType)>;

    /** @brief the callback will be called with the event's name as arg input */
    using ReceiveMsgCallback = std::function<void(const ara::core::String&)>;

    RtfEventHz();
    ~RtfEventHz() = default;

    RtfEventHz(const RtfEventHz& other) = delete;
    RtfEventHz& operator=(const RtfEventHz& other) & = delete;
    RtfEventHz(RtfEventHz&& other) = delete;
    RtfEventHz& operator=(RtfEventHz&& other) & = delete;

    bool Init(const ara::core::Vector<ara::core::String>& eventList, const uint32_t window,
              const HzOptions& hzOptions, HzInfoCallback callback);
    /**
     * @brief register a callback, which will be called everytime when we
     * receive msg from any interested events
     * @param eventList the interested event names
     * @param callback the callback will be called when receive msg
     * @note You shall not do any time consuming operations in the callback
     */
    bool Init(const ara::core::Vector<ara::core::String>& eventList,
              const HzOptions& hzOptions, ReceiveMsgCallback callback);
    void RtfEventHzStop();
    void RtfEventHzStart();
private:
    using SubResource = std::pair<rtf::com::Subscriber, std::unique_ptr<rtf::com::ThreadGroup>>;
    enum class InitState : std::uint8_t {
        NOT_INITED,
        RAW_CALLBACK,
        ONE_SECOND_CALLBACK
    };

    struct DDSEventIndexInfo {
        rtf::stdtype::uint16_t serviceId;
        rtf::stdtype::uint16_t instanceId;
        rtf::stdtype::String networkSegment;
        ara::core::String topic;
        rtf::stdtype::uint16_t domainId;
    };

    struct SOMEIPEventIndexInfo {
        rtf::stdtype::uint16_t serviceId;
        rtf::stdtype::uint16_t instanceId;
        rtf::stdtype::String networkSegment;
        rtf::stdtype::uint16_t eventId;
    };

    bool IsRepeatedEvent(const rtf::maintaind::EventInfoWithPubSub& eventInfoWithPubSub,
                         const ara::core::String& event) const;
    void DealWithSubscribedEvent(const ara::core::String& event,
        const rtf::maintaind::EventInfoWithPubSub& eventInfoWithPubSub,
        const bool ddsExist,
        const bool someipExist);
    bool IsUsingRawData(const rtf::maintaind::VectorTransportQos& transportQos) const noexcept;
    bool InitCommon(const ara::core::Vector<ara::core::String>& eventList, const HzOptions& hzOptions);
    void RtfEventPrintHz();
    bool RtfEventHzReady() const noexcept;
    void RtfEventGetHz();
    double GetMeanTimes(const std::deque<uint64_t>& times) const noexcept;
    double GetStdDevTimes(double mean, const std::deque<uint64_t>& times) const noexcept;
    void ReceivedMessageCallback(const ara::core::String& eventName);
    void SubscribeEvent(const ara::core::String& event);
    void UnSubscribe(const ara::core::String& event);
    bool ShouldSubscribe(const ara::core::String& event) noexcept;
    void PrintInfo(rtf::maintaind::DriverType driverType, const ara::core::String& event);
    void Subscribe();
    bool IsSubCreateFailed(const ara::core::String& event) const noexcept;
    bool IsSubscribed(const ara::core::String& event) const noexcept;
    void DoSubscribe(const ara::core::String& event,
        const ara::core::Vector<rtf::maintaind::EventInfoWithPubSub>& eventInfoWithPubSubList);
    bool CheckMethodEntity(const rtf::maintaind::EventInfoWithPubSub& info);
    SubResource SubscribeEvent(const ara::core::String& event, const bool isUsingRawData,
        const bool isUsingRecvMemory);
    void RegistConfig(const ara::core::String& event,
        const ara::core::Vector<rtf::maintaind::EventInfoWithPubSub>& eventInfoWithPubSubList,
        const bool ddsExist, const bool someipExist);
    void CheckSub(const ara::core::String& event, bool ddsExist, bool someipExist);
    bool IsAllEventsQueried() const noexcept;
    void QueryEvent() noexcept;
    bool IsQueried(const ara::core::String& eventName) const noexcept;

    HzOptions hzOption_;
    ara::core::Vector<ara::core::String> eventList_;
    std::set<ara::core::String> eventInfo_;     // 用来保存event的信息，可能存在同时查找多个event的场景
    ara::core::Vector<ara::core::String> foundEvent_;
    ara::core::Map<ara::core::String, DDSTopicConfig> subEventInfo_;
    std::set<ara::core::String> currentlyRecording_;
    std::set<ara::core::String> subCreateFailed_;
    ara::core::Vector<rtf::rtfevent::RtfEventInfo> eventInfoList_;
    ara::core::Map<ara::core::String, ara::core::String> types_;
    ara::core::Map<ara::core::String, SubResource> subs_;
    ara::core::Map<ara::core::String, DDSEventIndexInfo> ddsSubs_;
    ara::core::Map<ara::core::String, SOMEIPEventIndexInfo> someipSubs_;
    uint32_t window_;
    ara::core::Map<ara::core::String, EventHzInfo> eventHz_;  // hz信息保存表格，用来输出当前的信息
    std::multimap<ara::core::String, EventHzInfo> multiEventHz_;  // hz信息保存表格，用来输出当前的信息
    std::atomic<bool> eventHzEnable_;         // rtf event hz start stop控制
    bool ddsCreateError_;
    bool someipCreateError_;
    InitState initState_;
    std::mutex mutex_;
    ara::core::Map<ara::core::String, std::chrono::steady_clock::time_point> lastPrintTn_;
    ara::core::Map<ara::core::String, std::chrono::steady_clock::time_point> lastTimes_;
    ara::core::Map<ara::core::String, std::deque<uint64_t>> times_;   // event 消息间隔
    std::shared_ptr<rtf::com::NodeHandle> nh_;
    HzInfoCallback eventHzCallback_;     // 回调函数保存
    ReceiveMsgCallback receiveMsgCallback_;
    ara::core::Map<ara::core::String, uint32_t> filterEventMsg_;
    std::shared_ptr<rtf::rtftools::common::ToolsCommonClientManager> toolsCommonClientManager_;
    std::shared_ptr<rtftools::utils::NetProcess> netProcessPtr_;
};
}
}
#endif // RTFTOOLS_RTFEVENT_H
