/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
 * Description: rtf event latency API header
 * Create: 2020-12-22
 * Notes: N/A
 */

#ifndef RTFTOOLS_RTFEVENT_LATENCY_H
#define RTFTOOLS_RTFEVENT_LATENCY_H

#include <memory>
#include <mutex>
#include <string>
#include <vector>
#include <tuple>

#include "ara/core/map.h"
#include "ara/core/string.h"
#include "rtf/com/rtf_com.h"
#include "rtf/internal/tools_common_client_manager.h"
#include "rtf/rtfevent/RtfEvent.h"

namespace rtf {
namespace rtfevent {
using LatencyResult = rtf::maintaind::LatencyResult;
using LatencyResultMap = ara::core::Map<ara::core::String, rtf::rtfevent::LatencyResult>;
using LatencyMode = rtf::maintaind::LatencyMode;
struct EventLatencyStatus {
    ara::core::String pubApplicationName;
    rtf::rtfevent::LatencyMode pubStatus;
    ara::core::Map<ara::core::String, rtf::rtfevent::LatencyMode> subStatus;
};
class RtfEventLatency {
public:
    RtfEventLatency();
    ~RtfEventLatency() = default;

    RtfEventLatency(const RtfEventLatency& other) = delete;
    RtfEventLatency& operator=(const RtfEventLatency& other) & = delete;
    RtfEventLatency(RtfEventLatency&& other) = delete;
    RtfEventLatency& operator=(RtfEventLatency&& other) & = delete;

    int32_t Init();
    int EnableLatencyMode(const ara::core::String& eventName, 
                          const std::unique_ptr<CommonFilter> filter = nullptr);
    int QueryLatencyStatus(const ara::core::String& eventName, 
                           rtf::rtfevent::EventLatencyStatus& status, 
                           const std::unique_ptr<CommonFilter> filter = nullptr);
    int QueryLatencyResult(const ara::core::String& eventName, 
                           const std::uint16_t& timeWindows, 
                           LatencyResultMap& resultMap, 
                           const std::unique_ptr<CommonFilter> filter = nullptr);

    int DisableLatencyMode(const ara::core::String& eventName, 
                           const std::unique_ptr<CommonFilter> filter = nullptr);

private:
    static constexpr std::size_t FIND_RESULT {0U};
    static constexpr std::size_t ONLINE_RESULT {1U};
    static constexpr std::size_t LATENCY_INDEX {2U};
    static constexpr std::uint32_t WAIT_METHOD_REQUEST {300U};
    bool isInit_;
    bool hasIndex_;
    rtf::maintaind::LatencyIndex latencyIndex_;
    std::mutex initMutex_;
    std::shared_ptr<rtf::rtftools::common::ToolsCommonClientManager> toolsCommonClientManager_;
    std::shared_ptr<rtftools::utils::NetProcess> netProcessPtr_;

    int32_t SwitchLatencyMode(const ara::core::String& eventName, const rtf::maintaind::LatencyMode mode, const CommonFilter& filter);
    std::tuple<bool, bool, rtf::maintaind::LatencyIndex> QueryIndex(const ara::core::String& eventName, const CommonFilter& filter);
    ara::core::Vector<rtf::maintaind::EventInfoWithPubSub> FilterEventEntity(
        const ara::core::Vector<rtf::maintaind::EventInfoWithPubSub>& infoList, 
        const CommonFilter& filter) const;
    ara::core::Vector<rtf::maintaind::EventInfoWithPubSub> QueryEventInfo(const ara::core::String& eventName);
    ara::core::Vector<rtf::maintaind::EventInfoWithPubSub> FilterByInstanceName(
        const ara::core::Vector<rtf::maintaind::EventInfoWithPubSub>& eventInfoWithPubSubList,
        const ara::core::String& pubName) const;
};
}
}

#endif
