/* Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 * Description: rtf field API
 * Create: 2023-11-20
 * Notes: NA
 */

#ifndef RTFTOOLS_RTFFIELD_H
#define RTFTOOLS_RTFFIELD_H
#include <memory>
#include "rtf/function/RtftoolsReturnCode.h"
#include "rtf/internal/rtf_tools_type.h"
namespace rtf{
using namespace rtf::rtftools;
namespace rtffield{
using DataTypeReturnValue = rtf::DataTypeReturnValue;
using FieldShowReturnValue = rtf::ShowReturnValue;
using NotifyResourceInfo = rtf::rtfevent::ResourceInfo;

class RtfFieldInfoImpl;
class RtfFieldInfo {
public:
    RtfFieldInfo();
    void SetFieldName(const ara::core::String &fieldName);
    const ara::core::String GetFieldName() const noexcept;

    void SetFieldType(const ara::core::String &fieldType);
    const ara::core::String GetFieldType() const noexcept;

    void SetInstanceShortName(const ara::core::String &instanceShortName);
    const ara::core::String GetInstanceShortName() const noexcept;

    void SetServerName(const ara::core::String &serverName);
    const ara::core::String GetServerName() const noexcept;

    void SetClientList(const ara::core::Vector<ara::core::String> &clientList);
    const ara::core::Vector<ara::core::String> GetClientList() const noexcept;

    void SetUuid(const ara::core::String &uuid);
    const ara::core::String GetUuid() const noexcept;

    void SetNetworkIP(const ara::core::String &networkIP);
    const ara::core::String GetNetworkIP() const noexcept;

    void SetDomainId(const ara::core::String& domainId);
    const ara::core::String GetDomainId() const noexcept;

    void SetServiceId(const uint16_t serviceId);
    std::uint16_t GetServiceId() const noexcept;

    void SetInstanceId(const ara::core::String & instanceId);
    const ara::core::String GetInstanceId() const noexcept;

    void SetNetworkBinding(const ara::core::String &networkBind);
    const ara::core::String GetNetworkBinding() const noexcept;

    void SetNotifyTransportPlugin(const ara::core::String &transportPlugin);
    const ara::core::String GetNotifyTransportPlugin() const;

    void SetSetterTransportPlugin(const ara::core::String &transportPlugin);
    const ara::core::String GetSetterTransportPlugin() const;

    void SetGetterTransportPlugin(const ara::core::String &transportPlugin);
    const ara::core::String GetGetterTransportPlugin() const;

    void SetNotifyState(const bool &hasNotify);
    bool HasNotify() const noexcept;

    void SetSetterState(const bool &hasSetter);
    bool HasSetter() const noexcept;

    void SetGetterState(const bool &hasGetter);
    bool HasGetter() const noexcept;

    NotifyResourceInfo GetNotifyIAMResourceInfo() const noexcept;
    void SetNotifyIAMResourceInfo(const NotifyResourceInfo &resourceInfo) noexcept;

    ara::core::String GetSetterIAMResourceInfo() const noexcept;
    void SetSetterIAMResourceInfo(const ara::core::String &resourceInfo) noexcept;

    ara::core::String GetGetterIAMResourceInfo() const noexcept;
    void SetGetterIAMResourceInfo(const ara::core::String &resourceInfo) noexcept;
private:
    std::shared_ptr<RtfFieldInfoImpl> impl_;
};

class RtfFieldImpl;
class RtfField {
public:
    RtfField();
    ~RtfField() = default;

    RtfField(const RtfField& other) = default;
    RtfField& operator=(const RtfField& other) & = default;
    RtfField(RtfField&& other) = default;
    RtfField& operator=(RtfField&& other) & = default;

    RETURN_CODE Init();

    RETURN_CODE Query(const ara::core::String &fieldName,
                  RtfFieldInfo &rtfFieldInfo,
                  const std::unique_ptr<CommonFilter> &filter = nullptr) const;

    RETURN_CODE Query(const ara::core::String &fieldName,
                  ara::core::Vector<RtfFieldInfo> &rtfFieldInfoList,
                  const std::unique_ptr<CommonFilter> &filter = nullptr) const;

    RETURN_CODE QueryAll(ara::core::Vector<RtfFieldInfo> &rtfFieldInfoList) const;

    RETURN_CODE QueryAllWithNamespace(const ara::core::String &nameSpace,
                                  ara::core::Vector<RtfFieldInfo> &rtfFieldInfoList) const;

    RETURN_CODE QueryFieldShowInfo(const ara::core::String& fieldName,
                               FieldShowReturnValue &rtfFieldDataType,
                               const std::unique_ptr<CommonFilter> &filter = nullptr);
    RETURN_CODE QueryDataTypeInfo(const ara::core::String& dataTypeName,
                              ara::core::Vector<DataTypeReturnValue> &rtfDataTypeMsg) const;

    int32_t Export(const ara::core::String& fileName, const ara::core::String& path = "") const;

private:
    std::shared_ptr<RtfFieldImpl> impl_;
};
}
}
#endif //RTFTOOLS_RTFFIELD_H
