/* Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 * Description: rtf field API
 * Create: 2023-11-20
 * Notes: NA
 */

#ifndef RTFTOOLS_RTFFIELDCALL_H
#define RTFTOOLS_RTFFIELDCALL_H
#include <memory>
#include "rtf/function/RtftoolsReturnCode.h"
#include "rtf/internal/rtf_tools_type.h"
namespace rtf {
namespace rtffield {
using namespace rtf::rtftools;
struct EchoCallOptions {
    ara::core::String fieldName;
    ara::core::String ddsNet;
    ara::core::String someipNet;
    bool isUntypeset {false};
    bool isNoarr {false};
    ara::core::String uuid {""};
    std::uint16_t domainId {RTF_DEFAULT_DOMAINID};
    ara::core::String networkIP {""};
    bool echoImmediately_ {true};
};

class SampleInfo {
public:
    enum class StatusCode : uint8_t{
        SUCCESS = 0,
        UNDEFINED_ERROR,    // deserialize failed
        MISSING_MSG_INFO,   // deserialize failed for miss msg info of dataType
    };
    SampleInfo() : resultStr_(),
                   e2eResultStr_(), statusCode_(StatusCode::SUCCESS) {}
    ~SampleInfo() = default;
    SampleInfo(const SampleInfo& other) = delete;
    SampleInfo& operator=(const SampleInfo& other) & = delete;

    SampleInfo(SampleInfo && other) = default;
    SampleInfo& operator=(SampleInfo && other) & = default;
    ara::core::String& GetResult() { return resultStr_; }
    void SetResult(ara::core::String && result)
    {
        resultStr_ = std::move(result);
    }
    ara::core::String GetE2EResult() const {  return e2eResultStr_; }
    void SetE2EResult(ara::core::String && e2eResult) { e2eResultStr_ = std::move(e2eResult); }
    StatusCode GetStatusCode() const { return statusCode_; }
    void SetStatusCode(const StatusCode &code) { statusCode_ = code; }
    ara::core::String& GetDeserializationDataType() { return typeInfo_; }
    void SetDeserializationDataType(ara::core::String && typeInfo) { typeInfo_ = std::move(typeInfo); }
private:
    ara::core::String resultStr_;
    ara::core::String e2eResultStr_;
    StatusCode statusCode_;
    ara::core::String typeInfo_;
};
struct RtfFieldSetParams {
    ara::core::String inputParamJsonPath;
    ara::core::String inputParamsValue;
};

class RtfFieldEchoCallImpl;
class RtfFieldEchoCall {
public:
    using FieldEchoCallback = std::function<void(SampleInfo &&)>;
    using FieldSetCallback = std::function<void(SampleInfo &&)>;
    using FieldGetCallback = std::function<void(SampleInfo &&)>;

    RtfFieldEchoCall(void);
    ~RtfFieldEchoCall(void) = default;

    RtfFieldEchoCall(const RtfFieldEchoCall& other) = delete;
    RtfFieldEchoCall& operator=(const RtfFieldEchoCall& other) & = delete;
    RtfFieldEchoCall(RtfFieldEchoCall&& other) = delete;
    RtfFieldEchoCall& operator=(RtfFieldEchoCall&& other) & = delete;

    RETURN_CODE Init(const EchoCallOptions& echoCallOptions);
    void Stop();
    RETURN_CODE Echo(const FieldEchoCallback& fieldEchoCallback, bool isOnlyNotify = false);
    RETURN_CODE Get(const FieldGetCallback& fieldGetCallback, bool isOnlyGetter = false);
    RETURN_CODE Set(const RtfFieldSetParams &param, const FieldSetCallback& fieldSetCallback, bool isOnlySetter = false);

private:
    std::shared_ptr<RtfFieldEchoCallImpl> impl_;
};
}
}
#endif //RTFTOOLS_RTFFIELDCALL_H
