/* Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 * Description: rtf field API
 * Create: 2023-9-1
 * Notes: NA
 */

#ifndef RTFTOOLS_RTFFIELDHZ_H
#define RTFTOOLS_RTFFIELDHZ_H
#include <memory>
#include "rtf/function/RtftoolsReturnCode.h"
#include "rtf/com/rtf_com.h"
#include "rtf/internal/rtf_tools_type.h"
namespace rtf {
namespace rtffield {
using namespace rtf::rtftools;
// hz
struct FieldHzInfo {
    ara::core::String fieldName;
    double min_delta{0.0};   // 最小的时间间隔差。
    double max_delta{0.0};   // 最大的时间间隔差。
    double rate{0.0};        // 统计的平均频率，rate=num/sum。
    double std_dev{0.0};     // 所有时间标准差，标准差计算公式如下所示。
    uint32_t window{0};    // 记录消息窗口个数，有最大缓存上限，需要配置初始值。
    bool publish{false};       // 是否发布对应的消息
    bool isSubed{false};       // sub是否建立成功
    std::string uuid;
};

class SampleHzInfo {
public:
    enum class StatusCode {
        SUCCESS,
        ERR_DDS_CREATED,
        ERR_SOMEIP_CREATED,
        ERR_ALL_CREATED,
    };
    SampleHzInfo() : statusCode_(StatusCode::SUCCESS) {}
    ~SampleHzInfo() = default;
    SampleHzInfo(const SampleHzInfo& other) = delete;
    SampleHzInfo& operator=(const SampleHzInfo& other) & = delete;
    SampleHzInfo(SampleHzInfo && other) = default;
    SampleHzInfo& operator=(SampleHzInfo && other) & = default;

    void SetFieldHzInfoVec(ara::core::Vector<FieldHzInfo>&& fieldHzInfoVec) { fieldHzInfoVec_ = std::move(fieldHzInfoVec); }
    ara::core::Vector<FieldHzInfo>& GetFieldHzInfo() { return fieldHzInfoVec_; }

    void SetStatusCode(const StatusCode& statusCode) { statusCode_ = statusCode; }
    StatusCode GetStatusCode() const { return statusCode_; }
private:
    ara::core::Vector<FieldHzInfo> fieldHzInfoVec_;
    StatusCode statusCode_;
};

struct HzOptions {
    ara::core::String ddsNet{""};
    ara::core::String someipNet{""};
    std::uint32_t windowSize{50000U};   // default 50000
    ara::core::String uuid{""};
    ara::core::String networkIP{""};
    std::uint16_t domainId{RTF_DEFAULT_DOMAINID};
    bool hzImmediately_ {true};
};

class RtfFieldHzImpl;
class RtfFieldHz {
public:
    using HzInfoCallback = std::function<void(SampleHzInfo&&)>;
    RtfFieldHz();
    ~RtfFieldHz() = default;
    RtfFieldHz(const RtfFieldHz &other) = delete;
    RtfFieldHz& operator=(const RtfFieldHz& other) & = delete;
    RtfFieldHz(RtfFieldHz&& other) = delete;
    RtfFieldHz& operator=(RtfFieldHz&& other) & = delete;

    RETURN_CODE Init(const ara::core::Vector<ara::core::String>& fieldList, const HzOptions& hzOptions, const HzInfoCallback& hzInfoCallback);

    void Start();

    void Stop();

private:
    std::shared_ptr<RtfFieldHzImpl> impl_;
};
}
}
#endif //RTFTOOLS_RTFFIELDHZ_H
