/* Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 * Description: rtf method API
 * Create: 2020-10-27
 * Notes: NA
 */

#ifndef RTFTOOLS_RTFMETHOD_H
#define RTFTOOLS_RTFMETHOD_H

#include "ara/core/string.h"
#include "ara/core/vector.h"
#include "json_parser/document.h"
#include "rtf/internal/tools_common_client_manager.h"
#include "rtf/internal/rtf_tools_type.h"
#include "rtf/rtfnode/RtfNode.h"

namespace rtf {
namespace rtfmethod {
const ara::core::String DATA_TYPE_VALUE_UINT8 {"uint8_t"};
const ara::core::String DATA_TYPE_VALUE_UINT16 {"uint16_t"};
const ara::core::String DATA_TYPE_VALUE_UINT32 {"uint32_t"};
const ara::core::String DATA_TYPE_VALUE_UINT64 {"uint64_t"};
const ara::core::String DATA_TYPE_VALUE_INT8 {"int8_t"};
const ara::core::String DATA_TYPE_VALUE_INT16 {"int16_t"};
const ara::core::String DATA_TYPE_VALUE_INT32 {"int32_t"};
const ara::core::String DATA_TYPE_VALUE_INT64 {"int64_t"};
const ara::core::String DATA_TYPE_VALUE_STRING {"string"};
const ara::core::String DATA_TYPE_VALUE_FLOAT {"float"};
const ara::core::String DATA_TYPE_VALUE_DOUBLE {"double"};
const ara::core::String DATA_TYPE_STRUCT {"structure"};
const ara::core::String DATA_TYPE_VECTOR {"vector"};
const ara::core::String DATA_TYPE_ARRAY {"array"};
const ara::core::String DATA_TYPE_MAP {"associative_map"};
const ara::core::String DATA_TYPE_ENUMERATION {"enumeration"};
const ara::core::String DATA_TYPE_VARIANT {"variant"};

enum class RtfMethodRetType : int {
    SUCCESS = 0,
    FAILURE = -1,
    PARTIAL_FAILURE = -2
};

using DataTypeReturnValue = rtf::DataTypeReturnValue;

struct MethodShowReturnValue {
    ara::core::String dataTypeName;
    ara::core::String dataType;
    bool isOptional{false};
};
/**
 * the class to store basic method info
 * @example Name Example <br>
 * MethodName = ${MethodType}[${instanceShortname}] <br>
 * ServerName is ${AppName} also is ${Node}
 * @note instanceShortname could be empty
 */
class RtfMethodInfo {
public:
    RtfMethodInfo() = default;

    const ara::core::String GetMethodName() const;
    void SetMethodName(const ara::core::String& type2set);

    const ara::core::String& GetMethodType() const;
    void SetMethodType(const ara::core::String& type2set);

    const ara::core::Vector<ara::core::String> & GetRequestArgs() const;
    void SetRequestArgs(const ara::core::Vector<ara::core::String>& args);

    const ara::core::Vector<ara::core::String>& GetResponseArgs() const;
    void SetResponseArgs(const ara::core::Vector<ara::core::String>& args);

    const ara::core::String& GetServer() const;
    void SetServer(const ara::core::String& server);

    const ara::core::String& GetInstanceShortName() const;
    void SetInstanceShortName(const ara::core::String& instanceShortName);

    ara::core::String GetUuid() const { return uuid_; }
    void SetUuid(const ara::core::String &uuid) { uuid_ = uuid; }

    ara::core::String GetNetworkBinding() const { return networkBinding_; }
    void SetNetworkBinding(const ara::core::String &networkBinding) { networkBinding_ = networkBinding; }

    ara::core::String GetNetworkIP() const { return networkIP_; }
    void SetNetworkIP(const ara::core::String &networkIP) { networkIP_ = networkIP; }

    ara::core::String GetDomainId() const { return domainId_; }
    void SetDomainId(const ara::core::String &domainId) { domainId_ = domainId; }

    ara::core::String GetTransportPlugin() const { return transportPlugin_; }
    void SetTransportPlugin(const ara::core::String &transportPlugin) { transportPlugin_ = transportPlugin; }

    ara::core::Vector<ara::core::String> GetClients() const noexcept { return subList_; }
    void SetClients(const ara::core::Vector<ara::core::String>& subs) { subList_ = subs; }

    ara::core::String GetIAMResourceName() const noexcept { return iamResourceName_; };
    void SetIAMResourceName(const ara::core::String &resourceName) noexcept { iamResourceName_ = resourceName; };

    ara::core::Vector<ara::core::String> GetDiscoveryMulticastAddr() const { return discoveryMulticastAddr_; }
    void SetDiscoveryMulticastAddr(const ara::core::Vector<ara::core::String> &discoveryMulticastAddr) { discoveryMulticastAddr_ = discoveryMulticastAddr; }

private:
    ara::core::Vector<ara::core::String> subList_;
    /** methodName without instanceShortName */
    ara::core::String methodType_;
    /** equal to APP name */
    ara::core::String server_;
    /** instance name is surrounded by '[' ']' */
    ara::core::String instanceShortName_;
    ara::core::Vector<ara::core::String> reqArgs_;
    ara::core::Vector<ara::core::String> respArgs_;
    ara::core::String uuid_;
    ara::core::String networkBinding_;
    ara::core::String networkIP_;
    ara::core::String domainId_;
    ara::core::String transportPlugin_;
    ara::core::String iamResourceName_;
    ara::core::Vector<ara::core::String> discoveryMulticastAddr_;
};

class RtfMethod {
    using QueryMethodInfo = rtf::maintaind::proxy::methods::QueryMethodInfo;
    using QueryMethodType = rtf::maintaind::proxy::methods::QueryMethodType;
    using MethodRegisterInfo = rtf::maintaind::MethodRegisterInfo;
    using MethodInfoWithPubSub = rtf::maintaind::MethodInfoWithPubSub;
public:
    RtfMethod();
    int32_t Query(const ara::core::String methodName, RtfMethodInfo &rtfMethodInfo,
                  const std::unique_ptr<rtf::CommonFilter> filter = nullptr);
    int32_t Query(const ara::core::String methodName, ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList,
                  const std::unique_ptr<rtf::CommonFilter> filter = nullptr);
    int32_t QueryAll(ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList);
    int32_t QueryAllWithNamespace(const ara::core::String nameSpace, ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList);
    int32_t QueryMethodShowInfo(const ara::core::String methodName,
                            ara::core::Vector<MethodShowReturnValue> &rtfMethodReqDataTypeList,
                            ara::core::Vector<MethodShowReturnValue> &rtfMethodRepDataTypeList,
                            const std::unique_ptr<rtf::CommonFilter> filter = nullptr);
    int32_t QueryDataTypeInfo(const ara::core::String dataTypeName,
                              ara::core::Vector<DataTypeReturnValue> &rtfDataTypeMsg) const;
    int32_t Export(const ara::core::String& filename, const ara::core::String& path = "");
    int32_t Init();

private:
    bool RequestMethodData(const std::shared_ptr<rtf::rtftools::common::ToolsCommonClientManager> &clientManager,
                           ara::core::Vector<rtf::maintaind::MethodInfoWithPubSub> &methodInfoWithPubSubList) const;
    bool RequestNodeData(ara::core::Vector<rtf::rtfnode::RtfNodeInfo> &rtfNodeInfoList) const;
    bool GetShowResult(const QueryMethodType::Output &outPut,
                       ara::core::Vector<MethodShowReturnValue> &rtfMethodReqDataTypeList,
                       ara::core::Vector<MethodShowReturnValue> &rtfMethodRepDataTypeList) const;
    void GetInfoResult(const QueryMethodInfo::Output &outPut,
                       ara::core::Vector<MethodInfoWithPubSub> &methodInfoWithPubSubListTmp) const;
    void GetListResult(const ara::core::Vector<MethodInfoWithPubSub> &methodInfoWithPubSubList,
        ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList) const;
    void FilterMethodList(const ara::core::String &pubName,
                          const ara::core::Vector<MethodInfoWithPubSub> &methodInfoWithPubSubList,
                          const CommonFilter& filter,
                          ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList) const;
    void FilterResult(const ara::core::String &filtNamespace,
                      const ara::core::Vector<MethodInfoWithPubSub> &methodInfoWithPubSubList,
                      ara::core::Vector<RtfMethodInfo> &rtfMethodInfoList) const;

    void GetMethodTopStructInfo(const ara::core::String &topStructName,
        ara::core::Vector<MethodShowReturnValue> &rtfMethodDataTypeList) const;
    int32_t QueryAllDataType();
    bool isInit_ = false;
    std::shared_ptr<rtf::rtftools::common::ToolsCommonClientManager> toolsCommonClientManager_ = nullptr;
};
}
}
#endif // RTFTOOLS_RTFMETHOD_H
